@@ -2,6 +2,8 @@ from collections.abc import ValuesView
 from enum import Enum
 
 import pulp  # type: ignore
+import warnings  # We use this to ignore one specific warning from pulp
+
 
 from kep_solver.entities import Instance, Donor
 from kep_solver.graph import (
@@ -179,14 +181,42 @@ class BackArcs(Objective):
         return Sense.MAX
 
 
-class UKScore(Objective):
-    """An objective to maximise the number of back-arcs."""
+def UK_age_score(d1: Donor, d2: Donor) -> tuple[float, float]:
+    """Calculate and return the age difference bonus and tiebreaker value for a
+    given transplant where d1 is a Donor donating to the paired recipient of
+    Donor d2. The age difference bonus is 3, if the ages of the two donors
+    differ by at most 20, and the tie breaker is given by the equation
 
+      (70 - age_difference)^2 * 1e-5
+
+    Note that as a recipient may have multiple donors, this value is dependant
+    not only on a Transplant in an Exchange, but also which Transplant will
+    occur next. If d1 is a non-directed Donor, both the age difference bonus
+    and the tiebreaker will be 0.
+
+    :param d1: The Donor donating to the paired recipient of d2
+    :param d2: A paired donor of d1
+    :return: A tuple containing the age difference bonus, and the tiebreaker value.
+    """
     _age_weight_selector = 20
     _age_weight = 3
     _age_diff_factor = 1e-5
     _max_age_diff = 70
 
+    if d1.NDD:
+        return 0, 0
+    bonus = 0
+    age_diff = abs(d1.age - d2.age)
+    tb = _age_diff_factor * ((_max_age_diff - age_diff) ** 2)
+    tb = round(tb, 5)
+    if age_diff <= _age_weight_selector:
+        return _age_weight, tb
+    return 0, tb
+
+
+class UKScore(Objective):
+    """An objective to maximise the number of back-arcs."""
+
     def __init__(self):
         pass
 
@@ -209,11 +239,8 @@ class UKScore(Objective):
                 assert len(transplants) == 1
                 transplant = transplants[0]
                 score += transplant.weight
-                age_diff = abs(donor.age - target_v.donor.age)
-                if age_diff <= self._age_weight_selector:
-                    score += self._age_weight
-                age_diff_score = (self._max_age_diff - age_diff) ** 2
-                score += self._age_diff_factor * age_diff_score
+                bonus, tb = UK_age_score(donor, target_v.donor)
+                score += bonus + tb
         return score
 
     def describe(self) -> str:
@@ -349,9 +376,9 @@ class CycleAndChainModel(Model):
 
     @property
     def exchanges(self) -> list[Exchange]:
-        """Return the list of chains in this model.
+        """Return the list of cycles and chains in this model.
 
-        :return: the list of chains
+        :return: the list of cycles and chains
         """
         return list(self.chains) + list(self.cycles)
 
@@ -472,7 +499,11 @@ class CycleAndChainModel(Model):
             if not obj_equation:
                 self._objective_values.append(0)
                 continue
-            self += obj_equation
+            with warnings.catch_warnings():
+                warnings.filterwarnings(
+                    "ignore", "Overwriting previously set objective."
+                )
+                self += obj_equation
             self.sense = obj.sense.toObjective()
             solver = pulp.getSolver("PULP_CBC_CMD", msg=False)
             pulp.LpProblem.solve(self, solver)
