@@ -40,6 +40,15 @@ def test7_graph():
     return graph
 
 
+def test_model_build_1():
+    obj = model.TransplantCount()
+    instance = fileio.read_json("tests/test_instances/test1.json")
+    mod = model.CycleAndChainModel(instance, [obj], maxCycleLength=3, maxChainLength=3)
+    mod.build_model()
+    assert len(mod.cycles) == 2
+    assert len(mod.chains) == 0
+
+
 def test_transplant_count_test1(test1_graph):
     obj = model.TransplantCount()
     cycles = test1_graph.findCycles(3)
@@ -62,6 +71,41 @@ def test_transplant_count_test5(test5_graph):
             assert obj.value(test5_graph, chain) == 3
 
 
+def test_model_build_5():
+    obj = model.TransplantCount()
+    instance = fileio.read_json("tests/test_instances/test5.json")
+    mod = model.CycleAndChainModel(instance, [obj], maxCycleLength=3, maxChainLength=3)
+    mod.build_model()
+    assert len(mod.cycles) == 1
+    assert len(mod.chains) == 4
+
+
+def test_model_build_5b():
+    obj = model.TransplantCount()
+    instance = fileio.read_json("tests/test_instances/test5b.json")
+    mod = model.CycleAndChainModel(instance, [obj], maxCycleLength=3, maxChainLength=3)
+    mod.build_model()
+    assert len(mod.cycles) == 1
+    assert len(mod.chains) == 3
+    # Make sure all exchanges have different IDs
+    all_ids = set()
+    for e in mod.cycles:
+        assert e.id not in all_ids
+        all_ids.add(e.id)
+    for e in mod.chains:
+        assert e.id not in all_ids
+        all_ids.add(e.id)
+
+
+def test_model_build_8b():
+    obj = model.TransplantCount()
+    instance = fileio.read_json("tests/test_instances/test8b.json")
+    mod = model.CycleAndChainModel(instance, [obj], maxCycleLength=3, maxChainLength=3)
+    mod.build_model()
+    assert len(mod.cycles) == 9
+    assert len(mod.chains) == 0
+
+
 def test_effective_twoway_count_test1(test1_graph):
     obj = model.EffectiveTwoWay()
     cycles = test1_graph.findCycles(3)
